/*
 * Copyright 2006, Thomas Maier-Komor
 * LICENSE: GPLv2, see file COPYING for details
 */

#include "sysstat.h"

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <string.h>


static char *
per2str(double d)
{
	static char str[7];
	assert(d <= 1E2);
	if (d < 0) {
		memcpy(str,"n/a ",5);
	} else if (d < 2E-2) {
		sprintf(str,"%4.0f",d);
	} else if (d < 9.995) {
		sprintf(str,"%4.2f",d);
	} else if (d < 99.95) {
		sprintf(str,"%4.1f",d);
	} else
		sprintf(str,"%4.0f",rint(d));
	return str;
}


static char *
dbl2str(double d)
{
	static char str[7];
	double r = rint(d);
	if (d < 0) {
		memcpy(str,"unav.",6);
	} else if (d < 2E-2) {
		sprintf(str,"%4.0f ",d);
	} else if (d < 9.995) {
		sprintf(str,"%4.2f ",d);
	} else if (r < 1E2) {
		sprintf(str,"%4.1f ",d);
	} else if (r < 1E4) {
		sprintf(str,"%4.0f ",r);
	} else if (r < 1E5) {
		d /= 1E3;
		sprintf(str,"%4.1fk",d);
	} else if (d < 10E6) {
		d /= 1E3;
		sprintf(str,"%4.0fk",rint(d));
	} else if (d < 100E6) {
		d /= 1E6;
		sprintf(str,"%4.1fM",rint(d));
	} else {
		d /= 1E6;
		sprintf(str,"%4.0fM",rint(d));
	}
	return str;
}


static char *
kb2str(long long ll)
{
	static char str[8];
	char dim = ' ';
	double v;

	if (ll < 0) {
		memcpy(str,"???",4);
		return str;
	} else if (ll >= 10000LL*1024LL*1024LL) {
		v = (double)(ll >> 20)/1024;
		dim = 'T';
	} else if (ll >= 10000*1024) {
		v = (double)(ll >> 10)/1024;
		dim = 'G';
	} else if (ll >= 10000) {
		v = (double)ll/1024;
		dim = 'M';
	} else {
		v = (double) ll;
		dim = 'k';
	}
	if (rint(v) >= 100)
		snprintf(str,sizeof(str),"%4.0f%c",v,dim);
	else
		snprintf(str,sizeof(str),"%4.1f%c",v,dim);
	return str;
}


static char *
dkb2str(double v)
{
	static char str[8];
	char dim = ' ';

	if (v < 0) {
		memcpy(str,"???",4);
		return str;
	} else if (v >= 10000LL*1024LL*1024LL) {
		v /= 1024.0*1024.0*1024.0;
		dim = 'T';
	} else if (v >= 10000*1024.0) {
		v /= 1024*1024.0;
		dim = 'G';
	} else if (v >= 10000) {
		v /= 1024.0;
		dim = 'M';
	} else {
		dim = 'k';
	}
	if (rint(v) >= 100)
		snprintf(str,sizeof(str),"%4.0f%c",v,dim);
	else
		snprintf(str,sizeof(str),"%4.1f%c",v,dim);
	return str;
}


static void
display_sysstat(sysstat_t *s)
{
	userstat_t *u;
	fsstat_t *fs;
	char Time[32];

	if (s == 0)
		return;
	printf(s->Hostinfo);
	if (s->Ocpus == s->Ncpus) {
		if (s->Ncpus > 1) {
			printf(", %d CPUs", s->Ocpus);
		} else {
			assert(s->Ocpus == 1);
			printf(", 1 CPU");
		}
	} else {
		assert(s->Ncpus > 1);
		printf(", %d/%d CPUs",s->Ocpus,s->Ncpus);
	}
	if (s->Devices != 1) {
		printf(", %lu disks\t", s->Devices);
	} else {
		printf(", 1 disk\t");
	}
	strftime(Time,sizeof(Time),"%c",localtime(&s->TS));
	printf("%.20s\n",Time);

	/*-------------- CPU -------------*/
	printf(" User   CPU: %s%% | ", per2str(s->UserP));
	printf("LoadAv/1m:   %4.2f | ", s->loadavg[0]);
	printf("intr/s:     %s | ", dbl2str(s->IntrR));
	printf("pswitch/s:   %s \n", dbl2str(s->PswitchR));

	printf(" Kernel CPU: %s%% | ",per2str(s->KernelP));
	printf("LoadAv/5m:   %4.2f | ", s->loadavg[1]);
	printf("intrblk/s:  %s | ", dbl2str(s->IntrBlkR));
	printf("migrate/s:   %s \n", dbl2str(s->MigrateR));
	printf(" Idle   CPU: %s%%",per2str(s->IdleP));
	printf(" | LoadAv/15m:  %4.2f | ", s->loadavg[2]);
	printf("xcalls/s:   %s | ", dbl2str(s->XcallsR));
	printf("inv_swtch/s: %s \n", dbl2str(s->InvSwtchR));

	/*-------------- Mem -------------*/
	printf(" Physical:   %s | ", kb2str(s->Physmem));
	printf("SwapTotal:  %s | ", kb2str(s->SwapTotal));
	printf("Locked:     %s | ", kb2str(s->Pageslocked));
	printf("pgswapin/s:  %s \n", dbl2str(s->PgswapinR));

	printf(" Free:       %s | ", kb2str(s->Pagesfree));
	printf("SwapFree:   %s | ", kb2str(s->SwapFree));
	printf("scan/s:     %s | ",dbl2str(s->ScanR));
	printf("pgswapout/s: %s \n", dbl2str(s->PgswapoutR));

	/*-------------- I/O -------------*/
	printf(" syscall/s:  %s | ", dbl2str(s->SyscallR));
	printf("sysfork/s:  %s | ", dbl2str(s->SysforkR));
	printf("sysread/s:  %s | ", dbl2str(s->SysreadR));
	if (s->IpInR >= 0)
		printf("IP rcv/s:    %s ", dbl2str(s->IpInR));
	else
		printf("ipacket/s:   %s ", dbl2str(s->NetInR));

	printf(" nfsd ops/s: %s | ", dbl2str(s->NfsdR));
	printf("sysexec/s:  %s | ", dbl2str(s->SysexecR));
	printf("syswrite/s: %s | ", dbl2str(s->SyswriteR));
	if (s->IpOutR >= 0)
		printf("IP send/s:   %s ", dbl2str(s->IpOutR));
	else
		printf("opacket/s:   %s ", dbl2str(s->NetOutR));

	switch (s->Zpools) {
	default:
		printf(" %d Zpools:   %s | ", s->Zpools, kb2str(s->Zsize));
		break;
	case 1:
		printf(" %d Zpool:    %s | ", s->Zpools, kb2str(s->Zsize));
		break;
	case 0:
		printf(" Storage:    %s | ", kb2str(s->Storage));
		break;
	}
	printf("net errs: %6lu  | ", s->IpInDiscards+s->IpOutDiscards+s->IpInErrs);
	printf("DiskIn/s:   %s | ", dkb2str(s->DiskIR));
	printf("NfsIn/s:     %s \n", dkb2str(s->NfsIR));

	printf(" FsFree:     %s | ", kb2str(s->FsFree));
	printf("disk errs: %5lu  | ", s->HardErrors+s->SoftErrors);
	printf("DiskOut/s:  %s | ", dkb2str(s->DiskOR));
	printf("NfsOut/s:    %s \n", dkb2str(s->NfsOR));
	printf("--------------------------------------------------------------------------------\n");

	printf("User           NPROC ( O/ S/ R/ Z/ T) NLWP   SIZE    RSS  %MEM  %%CPU    CPU-TIME\n");
	u = s->us;
	while (u) {
		long h,m,s;
		assert(u->time.tv_sec >= 0);
		assert(u->time.tv_nsec >= 0);
		s = u->time.tv_sec;
		s += rint((double)u->time.tv_nsec/1000000000);
		m = s / 60;
		s -= m * 60;
		h = m / 60;
		m -= h * 60;
		printf("%-15s %4d (%2d/%2d/%2d/%2d/%2d) %4d  %s  ",
				u->username,
				u->nproc,
				u->oncpu,
				u->sleeping,
				u->runnable,
				u->zombie,
				u->stopped,
				u->nlwp,
				kb2str(u->size));
		printf("%s  %3.f%%  %3.f%%  %4d:%02d:%02d\n",
				kb2str(u->rss),
				u->pctmem,
				u->pctcpu,
				h,m,s
		       );
		u = u->next;
	}
	printf("--------------------------------------------------------------------------------\n");
	fs = s->fs;
	while (fs) {
		if (strlen(fs->dev) > 30)
			printf("%.27s...  %-4s ",fs->dev,fs->typ);
		else
			printf("%-30s  %-4s ",fs->dev,fs->typ);
		if (strlen(fs->mnt) > 30)
			printf("%.27s...  %s",fs->mnt,kb2str(fs->free));
		else
			printf("%-30s %s/",fs->mnt,kb2str(fs->free));
		printf("%s\n",kb2str(fs->blocks));
		fs = fs->next;
	}
	printf("================================================================================\n");
}


unsigned display_stdout(sysstat_t *s)
{
	unsigned numdis = 0;
	sysstat_t *f = s;
	if (s == 0)
		return 0;
	do {
		if (s->displayed == 0) {
			display_sysstat(s);
			s->displayed = 1;
			++numdis;
		}
		s = s->next;
	} while (s != f);
	return numdis;
}

// vim:tw=0
